#-----------------------------------------------------
# class Window_SubMission
# メニュー画面のミッションを表示するウィンドウクラス。
#-----------------------------------------------------

class Window_SubMission < Window_Selectable
  # 初期化
  def initialize(x, y, width, height)
    @missionlist = []
    @clear_icon = []
    make_mission_list
    super
  end
  # 行の高さ
  def line_height
    return 40
  end
  # 桁数の取得
  def col_max
    return 2
  end
  # 項目数の最大値の取得
  def item_max
    return [@missionlist.size, 1].max
  end
  # クリアアイコン用Imageの生成
  def create_clear_icon(idx, x, y)
    # すでにImageが生成されている場合はvisibleをtrueに
    @clear_icon[idx] = Sprite_Base.new(x + self.x + 500, y + self.y + 20)
    @clear_icon[idx].image = Image.load($game_directory + '/data/img/menu/clear.png')
  end
  # 全ての項目を描画
  def draw_all_items
    if @missionlist.size == 0
      rect = item_rect_for_text(0)
      draw_text(rect.x, rect.y, "\\C[9]Nothing")
    else
      super
    end
  end
  # 有効状態
  def enabled?(id)
    return $player.clear_sub_mission[id]
  end
  # コマンドリストの作成
  def make_mission_list
    @missionlist = []
    @missionlist = $player.sub_mission
  end
  # 項目の描画
  def draw_item(index)
    rect = item_rect_for_text(index)
    if enabled?(@missionlist[index].id) && window_range?(rect.y - @oy)
      create_clear_icon(index, rect.x + @ox, rect.y - @oy)
    else
      hide_clear_icon(index)
    end
    draw_text(rect.x, rect.y, @missionlist[index].name)
  end
  # クリアアイコンのyがウィンドウ内に収まっているか？
  def window_range?(y)
    y >= 0 && y < 420
  end
  # カーソルを下に移動
  def cursor_down
    if row < row_max - 1
      select(@index + col_max)
    else
      select(0)
    end
  end
  # カーソルを上に移動
  def cursor_up
    if row + 1 > 1
      select(@index - col_max)
    else
      select(item_max)
    end
  end
  # ヘルプテキスト更新
  def update_help
    if @missionlist.empty?
      @help_window.set_item(' ')
    else
      help = @missionlist[index].description
      @help_window.set_item(help.nil? ? ' ' : help)
    end
  end
  # リフレッシュ
  def refresh
    icon_dispose
    make_mission_list
    super
  end
  # 更新
  def update
    @clear_icon.each{|icon|
      unless icon.nil?
        icon.update
        icon.draw
      end
    }
    super
  end
  # 解放
  def dispose
    icon_dispose
    @missionlist = nil
    super
  end
  # 画面外になったクリアアイコンの非表示
  def hide_clear_icon(idx)
    @clear_icon[idx].visible = false unless @clear_icon[idx].nil?
  end
  # クリアアイコンの解放処理
  def icon_dispose
    @clear_icon.each{|icon| icon.image.dispose unless icon.nil?}
    @clear_icon.each{|icon|
      icon.dispose unless icon.nil?
      icon.vanish unless icon.nil?
    }
    @clear_icon = []
  end
end
